@@ -15,10 +15,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <glib.h>
+#include <gmp.h>
 #include <libguile.h>
 
 #include "../app.h"
 #include "../auth.h"
+#include "../auth_int.h"
 #include "../scheme.h"
 #include "../task.h"
 
@@ -31,12 +34,17 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
 {
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context (); 
-  gzochid_auth_identity *identity = gzochid_get_current_identity ();
-
+  gzochid_auth_identity *identity =
+    gzochid_auth_identity_clone (gzochid_get_current_identity ());
   gzochid_application_task *scheme_task = gzochid_application_task_new 
     (context, identity, gzochid_scheme_application_task_worker, callback);
+
+  /* FIXME: This object will stay protected even if the current transaction
+     gets rolled back. */
+
+  scm_gc_protect_object (callback);
   
-  if (scm_is_false (delay))
+  if (scm_is_false (delay) || scm_is_eq (delay, SCM_UNDEFINED))
     {
       gzochid_schedule_durable_task 
 	(context, identity, scheme_task, &gzochid_scheme_task_serialization);
@@ -53,7 +61,7 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
       delay_tv.tv_sec = d / 1000;
       delay_tv.tv_usec = (d % 1000) * 1000; 
 
-      if (scm_is_false (period))
+      if (scm_is_false (period) || scm_is_eq (period, SCM_UNDEFINED))
 	{
 	  gzochid_schedule_delayed_durable_task 
 	    (context, identity, scheme_task, 
@@ -93,6 +101,7 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
 SCM_DEFINE (primitive_cancel_task, "primitive-cancel-task", 1, 0, 0, 
 	    (SCM scm_handle), "Cancel a previously scheduled periodic task.")
 {
+  GError *err = NULL;
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context ();   
   gzochid_data_managed_reference *handle_reference = NULL;
@@ -105,13 +114,17 @@ SCM_DEFINE (primitive_cancel_task, "primitive-cancel-task", 1, 0, 0,
   handle_reference = gzochid_data_create_reference_to_oid 
     (context, &gzochid_durable_application_task_handle_serialization, oid);
 
-  gzochid_data_dereference (handle_reference);
-  
-  task_handle = (gzochid_periodic_task_handle *) handle_reference->obj;
-  gzochid_cancel_periodic_task (context, task_handle);
-
   mpz_clear (oid);
 
+  gzochid_data_dereference (handle_reference, &err);
+
+  if (err == NULL)
+    {
+      task_handle = (gzochid_periodic_task_handle *) handle_reference->obj;
+      gzochid_cancel_periodic_task (context, task_handle);
+    }
+  else gzochid_api_check_not_found (err);
+
   gzochid_api_check_transaction ();
 
   return SCM_UNSPECIFIED;
