@@ -28,9 +28,9 @@ struct _gzochid_game_context;
 gzochid_auth_identity *gzochid_auth_identity_clone (gzochid_auth_identity *);
 
 void gzochid_auth_identity_serializer 
-(struct _gzochid_application_context *, void *, GString *);
+(struct _gzochid_application_context *, void *, GString *, GError **);
 void *gzochid_auth_identity_deserializer
-(struct _gzochid_application_context *, GString *);
+(struct _gzochid_application_context *, GString *, GError **);
 void gzochid_auth_identity_finalizer
 (struct _gzochid_application_context *, void *);
 
