@@ -22,6 +22,7 @@
 #include <libintl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "admin.h"
 #include "config.h"
@@ -43,12 +44,13 @@
 #define Q(x) #x
 #define QUOTE(x) Q(x)
 
-#ifndef GZOCHI_CONF_LOCATION
-#define GZOCHI_CONF_LOCATION "/etc/gzochid.conf"
-#endif /* GZOCHI_CONF_LOCATION */
+#ifndef GZOCHID_CONF_LOCATION
+#define GZOCHID_CONF_LOCATION "/etc/gzochid.conf"
+#endif /* GZOCHID_CONF_LOCATION */
 
 static const struct option longopts[] =
   {
+    { "config", required_argument, NULL, 'c' },
     { "help", no_argument, NULL, 'h' },
     { "version", no_argument, NULL, 'v' },
     { NULL, 0, NULL, 0 }
@@ -64,13 +66,28 @@ static void initialize_async (gpointer data, gpointer user_data)
   GHashTable *game_config = NULL;  
   GHashTable *log_config = NULL;
 
+  GError *err = NULL;
+
   server_context->admin_context = 
     (gzochid_context *) gzochid_admin_context_new ();
   server_context->game_context = 
     (gzochid_context *) gzochid_game_context_new ();
 
   g_key_file_load_from_file 
-    (key_file, QUOTE(GZOCHID_CONF_LOCATION), G_KEY_FILE_NONE, NULL);
+    (key_file, server_context->gzochid_conf_path, G_KEY_FILE_NONE, &err);
+
+  if (err != NULL)
+    {
+      /* 
+	 The server has reasonable defaults for everything in gzochid.conf, so
+	 this condition is not fatal.
+      */
+
+      g_warning 
+	("Failed to load server configuration file %s: %s", 
+	 server_context->gzochid_conf_path, err->message);
+      g_error_free (err);
+    }
 
   admin_config = gzochid_config_keyfile_extract_config (key_file, "admin");
   game_config = gzochid_config_keyfile_extract_config (key_file, "game");
@@ -181,6 +198,7 @@ Usage: %s [OPTION]...\n"), program_name);
   
   puts ("");
   fputs (_("\
+  -c, --config        full path to gzochid.conf\n\
   -h, --help          display this help and exit\n\
   -v, --version       display version information and exit\n"), stdout);
 
@@ -222,9 +240,14 @@ int main (int argc, char *argv[])
 
   setlocale (LC_ALL, "");
 
-  while ((optc = getopt_long (argc, argv, "hv", longopts, NULL)) != -1)
+  context = gzochid_server_context_new ();
+
+  while ((optc = getopt_long (argc, argv, "c:hv", longopts, NULL)) != -1)
     switch (optc)
       {
+      case 'c':
+	context->gzochid_conf_path = strdup (optarg);
+	break;
       case 'v':
 	print_version ();
 	exit (EXIT_SUCCESS);
@@ -233,10 +256,14 @@ int main (int argc, char *argv[])
 	print_help (program_name);
 	exit (EXIT_SUCCESS);
 	break;
+      case '?': exit (EXIT_FAILURE);
       }
 
-  context = gzochid_server_context_new ();
+  if (context->gzochid_conf_path == NULL)
+    context->gzochid_conf_path = QUOTE(GZOCHID_CONF_LOCATION);
 
+  g_info ("Reading configuration from %s", context->gzochid_conf_path);
+  
   gzochid_server_context_init (context);
   gzochid_fsm_until (((gzochid_context *) context)->fsm, GZOCHID_STATE_STOPPED);
   gzochid_server_context_free (context);
