@@ -582,8 +582,9 @@ gboolean for_all (SCM lst, gboolean (*pred) (SCM))
   return TRUE;
 }
 
-static void scheme_managed_record_serializer
-(gzochid_application_context *context, SCM arg, GString *out)
+static void 
+scheme_managed_record_serializer
+(gzochid_application_context *context, SCM arg, GString *out, GError **err)
 {
   unsigned char vec_len_str[4];
   GList *gpd = g_list_append 
@@ -612,6 +613,8 @@ static void scheme_managed_record_serializer
       gzochid_transaction_mark_for_rollback 
 	(&scheme_participant, 
 	 is_transaction_retry (scm_variable_ref (exception_var)));
+      g_set_error (err, GZOCHID_IO_ERROR, GZOCHID_IO_ERROR_SERIALIZATION,
+		   "Failed to serialize managed record.");
       g_list_free (gpd);
       return;
     }
@@ -628,8 +631,9 @@ static void scheme_managed_record_serializer
   g_list_free (gpd);
 }
 
-static void *scheme_managed_record_deserializer
-(gzochid_application_context *context, GString *in)
+static void *
+scheme_managed_record_deserializer
+(gzochid_application_context *context, GString *in, GError **err)
 {
   int vec_len = gzochi_common_io_read_int ((unsigned char *) in->str, 0);
   SCM vec = SCM_EOL, port = SCM_EOL, record = SCM_EOL;
@@ -653,17 +657,29 @@ static void *scheme_managed_record_deserializer
   g_list_free (args);
   g_string_erase (in, 0, vec_len);
   
-  /* Don't join the transaction here as the serializer should only be
-     called during the PREPARING phase, which is too late to join. */
-  
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
       if (! triggered_by_rollback (scm_variable_ref (exception_var))) 
-	gzochid_transaction_mark_for_rollback 
-	  (&scheme_participant, 
-	   is_transaction_retry (scm_variable_ref (exception_var)));
+	{
+	  gzochid_transaction_join (&scheme_participant, NULL);
+	  gzochid_transaction_mark_for_rollback 
+	    (&scheme_participant, 
+	     is_transaction_retry (scm_variable_ref (exception_var)));
+	}
+
+      g_set_error (err, GZOCHID_IO_ERROR, GZOCHID_IO_ERROR_SERIALIZATION,
+		   "Failed to deserialize managed record.");
+    }
+  else 
+    {
+      /* If the bytevector port hasn't been fully consumed, it could mean the 
+	 serializer and deserializer aren't idempotent. */
+
+      if (scm_eof_object_p (scm_lookahead_u8 (port)) == SCM_BOOL_F)
+	gzochid_warning ("Deserialization failed to consume all bytes.");
+
+      scm_gc_protect_object (record);
     }
-  else scm_gc_protect_object (record);
 
   return record;
 }
@@ -674,20 +690,23 @@ static void *scheme_serializer_inner (void *data)
   gzochid_application_context *context = (gzochid_application_context *) ptr[0];
   SCM task = (SCM) ptr[1];
   GString *out = (GString *) ptr[2];
+  GError **err = (GError **) ptr[3];
 
-  scheme_managed_record_serializer (context, task, out);
+  scheme_managed_record_serializer (context, task, out, err);
 
   return NULL;
 }
 
-static void scheme_serializer
-(gzochid_application_context *context, void *ptr, GString *out)
+static void 
+scheme_serializer
+(gzochid_application_context *context, void *ptr, GString *out, GError **err)
 {
-  void *args[3];
+  void *args[4];
 
   args[0] = context;
   args[1] = ptr;
   args[2] = out;
+  args[3] = err;
 
   scm_with_guile (scheme_serializer_inner, args);
 }
@@ -697,17 +716,19 @@ static void *scheme_deserializer_inner (void *data)
   void **ptr = (void **) data;
   gzochid_application_context *context = (gzochid_application_context *) ptr[0];
   GString *in = (GString *) ptr[1];
+  GError **err = (GError **) ptr[2];
 
-  return scheme_managed_record_deserializer (context, in);
+  return scheme_managed_record_deserializer (context, in, err);
 }
 
 static void *scheme_deserializer
-(gzochid_application_context *context, GString *in)
+(gzochid_application_context *context, GString *in, GError **err)
 {
-  void *args[2];
+  void *args[3];
 
   args[0] = context;
   args[1] = in;
+  args[2] = err;
 
   return scm_with_guile (scheme_deserializer_inner, args);
 }
@@ -1038,9 +1059,9 @@ static void *initialize_bindings (void *ptr)
 	    "gzochi:make-channel");
   bind_scm ("gzochi private channel", &scm_channel_oid, "gzochi:channel-oid");
 
-  bind_scm ("gzochi client", &scm_handler_received_message,
+  bind_scm ("gzochi session", &scm_handler_received_message,
 	    "gzochi:client-session-listener-received-message");
-  bind_scm ("gzochi client", &scm_handler_disconnected,
+  bind_scm ("gzochi session", &scm_handler_disconnected,
 	    "gzochi:client-session-listener-disconnected");
 
   bind_scm ("gzochi conditions", &scm_make_name_exists_condition,
@@ -1065,9 +1086,9 @@ static void *initialize_bindings (void *ptr)
   bind_scm ("gzochi private data", &scm_managed_reference_oid, 
 	    "gzochi:managed-reference-oid");
 
-  bind_scm ("gzochi private client", &scm_make_client_session, 
+  bind_scm ("gzochi private session", &scm_make_client_session, 
 	    "gzochi:make-client-session");
-  bind_scm ("gzochi private client", &scm_client_session_oid, 
+  bind_scm ("gzochi private session", &scm_client_session_oid, 
 	    "gzochi:client-session-oid");
 
   bind_scm ("gzochi private data", &scm_make_managed_reference, 
@@ -1124,6 +1145,4 @@ void gzochid_scheme_task_handle_oid (SCM handle, mpz_t oid)
 void gzochid_scheme_initialize_bindings (void)
 {
   scm_with_guile (initialize_bindings, NULL);
-
-  gzochid_task_register_serialization (&gzochid_scheme_task_serialization);
 }
