@@ -1,5 +1,5 @@
 ;; gzochi/private/data.scm: Private infrastructure for gzochi data API
-;; Copyright (C) 2013 Julian Graham
+;; Copyright (C) 2014 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -24,12 +24,18 @@
 	  gzochi:managed-record-constructor
 	  gzochi:managed-record-predicate
 
-          gzochi:define-managed-record-type
+	  gzochi:make-managed-record-type-registry
+	  gzochi:managed-record-type-registry?
+	  gzochi:push-type-registry!
+	  gzochi:pop-type-registry!
+
+	  gzochi:define-managed-record-type
 	  gzochi:managed-record-type-descriptor
 	  gzochi:managed-record-constructor-descriptor
 
 	  gzochi:make-managed-record
 
+	  gzochi:managed-record
 	  gzochi:managed-record?
 	  gzochi:managed-record-rtd 
 	  gzochi:managed-record-type-name
@@ -59,7 +65,10 @@
 	  gzochi:mark-for-read!)
 
   (import (gzochi io)
-	  (only (guile) and=> gensym macroexpand) 
+	  (gzochi log)
+	  (only (guile) and=> fluid-ref fluid-set! gensym macroexpand 
+		        make-fluid simple-format) 
+	  (ice-9 optargs)
 	  (rnrs)
 	  (only (srfi :1) last split-at take)
 	  (srfi :8))
@@ -121,15 +130,10 @@
      gzochi:serialize-managed-reference 
      gzochi:deserialize-managed-reference))
   
-  (define (gzochi:make-managed-record-type-descriptor 
-	   name parent uid sealed? opaque? fields)
+  (define* (gzochi:make-managed-record-type-descriptor 
+	    name parent uid sealed? opaque? fields 
+	    #:key serial-uid type-registry)
     
-    (or uid 
-	(raise 
-	 (condition
-	  (make-assertion-violation)
-	  (make-message-condition "Managed records must be nongenerative"))))
-
     (let* ((fields (vector->list fields))
 	   (field-names (map (lambda (f) (take f 2)) fields))
 	   (field-serializations (map last fields))
@@ -140,15 +144,29 @@
 				(make-assertion-violation)
 				(make-message-condition
 				 "Parent types must be managed"))))))
+
+	   (serial-uid (or serial-uid uid name))
+	   (type-registry (or type-registry default-type-registry))
+
 	   (rtd (make-record-type-descriptor
 		 name managed-parent uid sealed? opaque? 
 		 (list->vector field-names))))
-      
-      (gzochi:register-record-type! uid rtd)
-      (gzochi:register-serialization! 
-       uid (list->vector field-serializations))
 
-      rtd))
+      (let ((reg (gzochi:serial-uid->type-registration 
+		  serial-uid type-registry)))
+	(if reg
+	    (or (eq? (gzochi:managed-record-type-registration-rtd reg) rtd)
+		(raise (condition
+			(make-assertion-violation)
+			(make-message-condition 
+			 "Managed records must be nongenerative"))))
+
+	    (gzochi:register-type! 
+	     type-registry
+	     (gzochi:make-managed-record-type-registration 
+	      serial-uid rtd (list->vector field-serializations))))
+	
+	rtd)))
 
   (define (gzochi:make-managed-record-constructor-descriptor 
 	   rtd parent-constructor-descriptor protocol)
@@ -196,14 +214,14 @@
 		  (apply field-binder
 			 (map maybe-create-reference p-args))))))))))
 
-  (define managed-record-type-registry (make-eq-hashtable))
+  (define managed-record-name-registry (make-eq-hashtable))
 
-  (define (register-managed-record-type name rtd rcd)
-    (hashtable-set! managed-record-type-registry name (cons rtd rcd)))
+  (define (register-managed-record-name name rtd rcd)
+    (hashtable-set! managed-record-name-registry name (cons rtd rcd)))
   (define (lookup-managed-record-type-descriptor name)
-    (and=> (hashtable-ref managed-record-type-registry name #f) car))
+    (and=> (hashtable-ref managed-record-name-registry name #f) car))
   (define (lookup-managed-record-constructor-descriptor name)
-    (and=> (hashtable-ref managed-record-type-registry name #f) cdr))
+    (and=> (hashtable-ref managed-record-name-registry name #f) cdr))
 
   (define (guess-constructor-name record-name)
     (string->symbol (string-append "make-" (symbol->string record-name))))
@@ -330,13 +348,16 @@
                     (_sealed *unspecified*)
                     (_opaque *unspecified*)
                     (_nongenerative *unspecified*)
+		    (_serial-uid *unspecified*)
+		    (_type-registry *unspecified*)
                     (_constructor *unspecified*)
                     (_parent-rtd *unspecified*)
                     (record-clauses #'(record-clause ...)))
 
            (syntax-case record-clauses
                (fields parent protocol sealed opaque nongenerative
-                       constructor parent-rtd serialization)
+                       serial-uid type-registry constructor parent-rtd 
+		       serialization)
              [()
               (let* ((fields (if (unspecified? _fields) '() _fields))
                      (processed-fields (process-fields #'record-name fields))
@@ -375,10 +396,12 @@
 			    ((not (unspecified? _parent-rtd))
 			     (car _parent-rtd))
 			    (else #f)))
+
                      (protocol (if (unspecified? _protocol) #f _protocol))
-                     (uid (if (unspecified? _nongenerative) 
-			      #''record-name 
-			      _nongenerative))
+                     (uid (if (unspecified? _nongenerative) #f _nongenerative))
+		     (serial-uid (if (unspecified? _serial-uid) #f _serial-uid))
+		     (type-registry 
+		      (if (unspecified? _type-registry) #f _type-registry))
                      (sealed? (if (unspecified? _sealed) #f _sealed))
                      (opaque? (if (unspecified? _opaque) #f _opaque)))
 
@@ -387,10 +410,12 @@
                       (gzochi:make-managed-record-type-descriptor
                        (quote record-name)
                        #,parent-rtd #,uid #,sealed? #,opaque? 
-		       (list->vector #,field-names)))
+		       (list->vector #,field-names)
+		       #:serial-uid #,serial-uid
+		       #:type-registry #,type-registry))
 		    
 		    (let ()
-		      (register-managed-record-type 
+		      (register-managed-record-name 
 		       (quote record-name)
 		       record-name 
 		       (gzochi:make-managed-record-constructor-descriptor 
@@ -406,46 +431,65 @@
              [((fields record-fields ...) . rest)
               (if (unspecified? _fields)
                   (loop #'(record-fields ...) _parent _protocol _sealed _opaque
-			_nongenerative _constructor _parent-rtd #'rest)
+			_nongenerative _serial-uid _type-registry _constructor 
+			_parent-rtd #'rest)
                   (raise (make-assertion-violation)))]
              [((parent parent-name) . rest)
               (if (not (unspecified? _parent-rtd))
                   (raise (make-assertion-violation))
                   (if (unspecified? _parent)
                       (loop _fields #'parent-name _protocol _sealed _opaque
-                            _nongenerative _constructor _parent-rtd #'rest)
+                            _nongenerative _serial-uid _type-registry 
+			    _constructor _parent-rtd #'rest)
                       (raise (make-assertion-violation))))]
              [((protocol expression) . rest)
               (if (unspecified? _protocol)
                   (loop _fields _parent #'expression _sealed _opaque
-                        _nongenerative _constructor _parent-rtd #'rest)
+                        _nongenerative _serial-uid _type-registry _constructor 
+			_parent-rtd #'rest)
                   (raise (make-assertion-violation)))]
              [((sealed sealed?) . rest)
               (if (unspecified? _sealed)
                   (loop _fields _parent _protocol #'sealed? _opaque
-                        _nongenerative _constructor _parent-rtd #'rest)
+                        _nongenerative _serial-uid _type-registry _constructor 
+			_parent-rtd #'rest)
                   (raise (make-assertion-violation)))]
              [((opaque opaque?) . rest)
               (if (unspecified? _opaque)
                   (loop _fields _parent _protocol _sealed #'opaque?
-                        _nongenerative _constructor _parent-rtd #'rest)
+                        _nongenerative _serial-uid _type-registry _constructor 
+			_parent-rtd #'rest)
                   (raise (make-assertion-violation)))]
              [((nongenerative) . rest)
               (if (unspecified? _nongenerative)
                   (loop _fields _parent _protocol _sealed _opaque 
-			#''record-name _constructor _parent-rtd #'rest)
+			#''record-name _serial-uid _type-registry _constructor 
+			_parent-rtd #'rest)
                   (raise (make-assertion-violation)))]
              [((nongenerative uid) . rest)
               (if (unspecified? _nongenerative)
                   (loop _fields _parent _protocol _sealed _opaque #''uid 
-			_constructor _parent-rtd #'rest)
+			_serial-uid _type-registry _constructor _parent-rtd 
+			#'rest)
                   (raise (make-assertion-violation)))]
+	     [((serial-uid uid) . rest)
+	      (if (unspecified? _serial-uid)
+		  (loop _fields _parent _protocol _sealed _opaque _nongenerative
+			#''uid _type-registry _constructor _parent-rtd #'rest)
+		  (raise (make-assertion-violation)))]
+	     [((type-registry registry) . rest)
+	      (if (unspecified? _type-registry)
+		  (loop _fields _parent _protocol _sealed _opaque _nongenerative
+			_serial-uid #'registry _constructor _parent-rtd
+			#'rest)
+		  (raise (make-assertion-violation)))]
              [((parent-rtd rtd cd) . rest)
               (if (not (unspecified? _parent))
                   (raise (make-assertion-violation))
                   (if (unspecified? _parent-rtd)
                       (loop _fields _parent _protocol _sealed _opaque
-                            _nongenerative _constructor #'(rtd cd) #'rest)
+                            _nongenerative _serial-uid _type-registry 
+			    _constructor #'(rtd cd) #'rest)
                       (raise (make-assertion-violation))))]))))))
 
   (define-syntax gzochi:managed-record-type-descriptor
@@ -463,13 +507,25 @@
 			stx (list 'quote (syntax->datum #'name))))))))
 
   (define (gzochi:serialize-managed-record port record)
-    (define (serialize rtd)
+    (define (lookup-registration rtd)
+      (or (gzochi:rtd->type-registration rtd)
+	  (raise (condition 
+		  (make-assertion-violation)
+		  (make-message-condition 
+		   (simple-format #f 
+				  "Attempting to serialize unknown type ~A" 
+				  (record-type-name rtd)))))))
+
+    (define (serialize rtd serialization)
       (let ((parent-rtd (record-type-parent rtd)))
 	(if (and parent-rtd (not (eq? parent-rtd gzochi:managed-record)))
-	    (serialize parent-rtd))
-	(let ((serialization 
-	       (gzochi:uid->serialization (record-type-uid rtd)))
-	      (num-fields (vector-length (record-type-field-names rtd))))
+	    (let ((parent-registration (lookup-registration parent-rtd)))
+	      (serialize 
+	       parent-rtd 
+	       (gzochi:managed-record-type-registration-field-serializations 
+		parent-registration))))
+
+	(let ((num-fields (vector-length (record-type-field-names rtd))))
 	  (let loop ((i 0))
 	    (and (< i num-fields)
 		 (let ((accessor (record-accessor rtd i))
@@ -479,17 +535,19 @@
 
 		   (serializer port (accessor record))
 		   (loop (+ i 1))))))))
-    
-    (let ((rtd (record-rtd record)))
-      (gzochi:write-symbol port (record-type-uid rtd))
-      (serialize rtd)))
+
+    (let* ((rtd (record-rtd record))
+	   (registration (lookup-registration rtd)))
+
+      (gzochi:write-symbol
+       port (gzochi:managed-record-type-registration-serial-uid registration))
+      (serialize 
+       rtd (gzochi:managed-record-type-registration-field-serializations 
+	    registration))))
 
   (define (gzochi:deserialize-managed-record port)
-    (define (deserialize rtd)
-      (let ((serialization 
-	     (gzochi:uid->serialization (record-type-uid rtd)))
-	    (num-fields (vector-length (record-type-field-names rtd))))
-
+    (define (deserialize rtd serialization)
+      (let ((num-fields (vector-length (record-type-field-names rtd))))
 	(let loop ((i 0) (vals '()))
 	  (if (< i num-fields)
 	      (let ((deserializer (or (and=> (vector-ref serialization i)
@@ -498,49 +556,130 @@
 		(loop (+ i 1) (cons (deserializer port) vals)))
 	      (reverse vals)))))
 	
-    (define (constructor-descriptor rtd)
-      (let* ((parent-rtd (record-type-parent rtd))
-	     (parent-rctd 
-	      (and parent-rtd 
-		   (not (eq? parent-rtd gzochi:managed-record))
-		   (constructor-descriptor parent-rtd))))
+    (define (constructor-descriptor rtd serialization)
+      (let* 
+	  ((parent-rtd (record-type-parent rtd))
+	   (parent-registration 
+	    (and parent-rtd 
+		 (not (eq? parent-rtd gzochi:managed-record))
+		 (or (gzochi:rtd->type-registration parent-rtd)
+		     (raise (condition 
+			     (make-assertion-violation)
+			     (make-message-condition 
+			      (string-append 
+			       "Attempting to deserialize unknown type " 
+			       (record-type-name parent-rtd))))))))
+	   
+	   (parent-rctd 
+	    (and parent-registration
+		 (constructor-descriptor 
+		  parent-rtd 
+		  (gzochi:managed-record-type-registration-field-serializations
+		   parent-registration)))))
+		  
 	(make-record-constructor-descriptor 
 	 rtd parent-rctd (lambda (n)
 			   (lambda ()
 			     (let ((p (n)))
-			       (apply p (deserialize rtd))))))))
+			       (apply p (deserialize rtd serialization))))))))
+	
     
-    (let* ((uid (gzochi:read-symbol port))
-	   (rtd (gzochi:uid->record-type uid)))
-      (or rtd (raise (condition 
-		      (make-assertion-violation)
-		      (make-message-condition 
-		       (string-append 
-			"Attempting to deserialize unknown type " 
-			(symbol->string uid))))))
+    (let* ((serial-uid (gzochi:read-symbol port))
+	   (registration (gzochi:serial-uid->type-registration serial-uid)))
+      (or registration 
+	  (raise (condition 
+		  (make-assertion-violation)
+		  (make-message-condition 
+		   (string-append 
+		    "Attempting to deserialize unknown type " 
+		    (symbol->string serial-uid))))))
+
+      (let* 
+	  ((rctd 
+	    (constructor-descriptor 
+	     (gzochi:managed-record-type-registration-rtd registration)
+	     (gzochi:managed-record-type-registration-field-serializations
+	      registration)))
+	   (ctor (record-constructor rctd)))
 
-      (let* ((rctd (constructor-descriptor rtd))
-	     (ctor (record-constructor rctd)))
 	(ctor))))
   
-  (define record-type-registry (make-eq-hashtable))
+  (define-record-type (gzochi:managed-record-type-registration
+		       gzochi:make-managed-record-type-registration
+		       gzochi:managed-record-type-registration?)
+    (fields serial-uid rtd field-serializations))
 
-  (define (gzochi:register-record-type! uid rtd)
-    (hashtable-set! record-type-registry uid rtd))
-  (define (gzochi:uid->record-type uid)
-    (hashtable-ref record-type-registry uid #f))
+  (define-record-type (gzochi:managed-record-type-registry 
+		       gzochi:make-managed-record-type-registry 
+		       gzochi:managed-record-type-registry?)
 
-  (define serializer-registry (make-eq-hashtable))
+    (fields rtds serial-uids)
 
-  (define (gzochi:register-serialization! uid serialization)
-    (hashtable-set! serializer-registry uid serialization))
-  (define (gzochi:uid->serialization uid) 
-    (hashtable-ref serializer-registry uid #f))
+    (protocol (lambda (p) 
+		(lambda () 
+		  (p (make-eq-hashtable) 
+		     (make-eq-hashtable)))))
+    (sealed #t))
+
+  (define default-type-registry (gzochi:make-managed-record-type-registry))
+  (define type-registry-stack (make-fluid '()))
+
+  (define (gzochi:push-type-registry! registry)
+    (fluid-set! type-registry-stack 
+		(cons registry (fluid-ref type-registry-stack))))
+
+  (define (gzochi:pop-type-registry!)
+    (let ((stack (fluid-ref type-registry-stack)))
+      (if (null? stack)
+	  (raise (condition 
+		  (make-assertion-violation)
+		  (make-message-condition "Type registry stack is empty")))
+	  (let ((registry (car stack))
+		(stack (cdr stack)))
+	    (fluid-set! type-registry-stack stack)
+	    registry))))
+
+  (define (gzochi:register-type! registry reg)
+    (hashtable-set! 
+     (gzochi:managed-record-type-registry-rtds registry)
+     (gzochi:managed-record-type-registration-rtd reg) reg)
+    (hashtable-set!
+     (gzochi:managed-record-type-registry-serial-uids registry) 
+     (gzochi:managed-record-type-registration-serial-uid reg) reg))
+
+  (define (gzochi:rtd->type-registration rtd . registry)
+    (define (rtd->type-registration-inner registry)
+      (hashtable-ref
+       (gzochi:managed-record-type-registry-rtds registry) rtd #f))
+    (define (rtd->type-registration-recursive registry-stack) 
+      (cond ((null? registry-stack)
+	     (rtd->type-registration-inner default-type-registry))
+	    ((rtd->type-registration-inner (car registry-stack)))
+	    (else (rtd->type-registration-recursive (cdr registry-stack)))))
+    (if (null? registry)
+	(rtd->type-registration-recursive (fluid-ref type-registry-stack))
+	(rtd->type-registration-inner (car registry))))
+
+  (define (gzochi:serial-uid->type-registration serial-uid . registry)
+    (define (serial-uid->type-registration-inner registry)
+      (hashtable-ref
+       (gzochi:managed-record-type-registry-serial-uids registry) serial-uid 
+       #f))
+    (define (serial-uid->type-registration-recursive registry-stack)
+      (cond ((null? registry-stack)
+	     (serial-uid->type-registration-inner default-type-registry))
+	    ((serial-uid->type-registration-inner (car registry-stack)))
+	    (else (serial-uid->type-registration-recursive
+		   (cdr registry-stack)))))
+    (if (null? registry)
+	(serial-uid->type-registration-recursive 
+	 (fluid-ref type-registry-stack))
+	(serial-uid->type-registration-inner (car registry))))
 
   (define (gzochi:mark-for-write! managed-record)
     (or (gzochi:managed-record? managed-record)
 	(raise (condition (make-assertion-violation)
-			  (make-message-condition 
+			  (make-message-condition
 			   "Only managed records may be marked."))))
     (primitive-mark-for-write! managed-record))
     
