@@ -25,6 +25,8 @@
 #include "../storage.h"
 #include "../tx.h"
 
+static GPrivate thread_application_context_key;
+
 gzochid_application_context *gzochid_application_context_new ()
 {
   gzochid_application_context *context = 
@@ -34,10 +36,12 @@ gzochid_application_context *gzochid_application_context_new ()
   g_mutex_init (&context->client_mapping_lock);
 
   context->storage_context = gzochid_storage_initialize ("/dev/null?");
-  context->meta = gzochid_storage_open (context->storage_context, "/dev/null");
-  context->oids = gzochid_storage_open (context->storage_context, "/dev/null");
+  context->meta = gzochid_storage_open 
+    (context->storage_context, "/dev/null", 0);
+  context->oids = gzochid_storage_open 
+    (context->storage_context, "/dev/null", 0);
   context->names = gzochid_storage_open 
-    (context->storage_context, "/dev/null");
+    (context->storage_context, "/dev/null", 0);
 
   context->event_source = gzochid_application_event_source_new ();
 
@@ -169,3 +173,33 @@ gzochid_transaction_result gzochid_application_transaction_execute_timed
 {
   gzochid_transaction_execute_timed (fn, data, timeout);
 }
+
+gzochid_application_callback *gzochid_application_callback_new
+(char *procedure, GList *module, mpz_t scm_oid)
+{
+  gzochid_application_callback *callback = calloc
+    (1, sizeof (gzochid_application_callback));
+
+  callback->module = module;
+  callback->procedure = procedure;
+
+  mpz_init (callback->scm_oid);
+  mpz_set (callback->scm_oid, scm_oid);
+
+  return callback;
+}
+
+gzochid_application_context *
+gzochid_get_current_application_context ()
+{
+  return (gzochid_application_context *)
+    g_private_get (&thread_application_context_key);
+}
+
+void *
+gzochid_with_application_context
+(gzochid_application_context *context, gzochid_auth_identity *auth,
+ void *(*worker) (gpointer), gpointer data)
+{
+  worker (data);
+}
